@@ -5,7 +5,6 @@ from esphome.const import (
     CONF_ID,
     CONF_RESOLUTION,
     DEVICE_CLASS_ILLUMINANCE,
-    ICON_EMPTY,
     STATE_CLASS_MEASUREMENT,
     UNIT_LUX,
     CONF_MEASUREMENT_DURATION,
@@ -28,7 +27,10 @@ BH1750Sensor = bh1750_ns.class_(
 CONF_MEASUREMENT_TIME = "measurement_time"
 CONFIG_SCHEMA = (
     sensor.sensor_schema(
-        UNIT_LUX, ICON_EMPTY, 1, DEVICE_CLASS_ILLUMINANCE, STATE_CLASS_MEASUREMENT
+        unit_of_measurement=UNIT_LUX,
+        accuracy_decimals=1,
+        device_class=DEVICE_CLASS_ILLUMINANCE,
+        state_class=STATE_CLASS_MEASUREMENT,
     )
     .extend(
         {
