@@ -1,19 +1,23 @@
 #pragma once
+
+#ifdef USE_ESP32
+
 #include "esphome/core/component.h"
 #include "esphome/core/helpers.h"
 
 #include <queue>
 #include <mutex>
-
-#ifdef ARDUINO_ARCH_ESP32
+#include <cstring>
 
 #include <esp_gap_ble_api.h>
 #include <esp_gatts_api.h>
 #include <esp_gattc_api.h>
+#include <freertos/FreeRTOS.h>
+#include <freertos/semphr.h>
 
 /*
  * BLE events come in from a separate Task (thread) in the ESP32 stack. Rather
- * than trying to deal wth various locking strategies, all incoming GAP and GATT
+ * than trying to deal with various locking strategies, all incoming GAP and GATT
  * events will simply be placed on a semaphore guarded queue. The next time the
  * component runs loop(), these events are popped off the queue and handed at
  * this safer time.
@@ -24,33 +28,33 @@ namespace esp32_ble {
 
 template<class T> class Queue {
  public:
-  Queue() { m = xSemaphoreCreateMutex(); }
+  Queue() { m_ = xSemaphoreCreateMutex(); }
 
   void push(T *element) {
     if (element == nullptr)
       return;
-    if (xSemaphoreTake(m, 5L / portTICK_PERIOD_MS)) {
-      q.push(element);
-      xSemaphoreGive(m);
+    if (xSemaphoreTake(m_, 5L / portTICK_PERIOD_MS)) {
+      q_.push(element);
+      xSemaphoreGive(m_);
     }
   }
 
   T *pop() {
     T *element = nullptr;
 
-    if (xSemaphoreTake(m, 5L / portTICK_PERIOD_MS)) {
-      if (!q.empty()) {
-        element = q.front();
-        q.pop();
+    if (xSemaphoreTake(m_, 5L / portTICK_PERIOD_MS)) {
+      if (!q_.empty()) {
+        element = q_.front();
+        q_.pop();
       }
-      xSemaphoreGive(m);
+      xSemaphoreGive(m_);
     }
     return element;
   }
 
  protected:
-  std::queue<T *> q;
-  SemaphoreHandle_t m;
+  std::queue<T *> q_;
+  SemaphoreHandle_t m_;
 };
 
 // Received GAP, GATTC and GATTS events are only queued, and get processed in the main loop().
@@ -101,11 +105,13 @@ class BLEEvent {
   };
 
   union {
+    // NOLINTNEXTLINE(readability-identifier-naming)
     struct gap_event {
       esp_gap_ble_cb_event_t gap_event;
       esp_ble_gap_cb_param_t gap_param;
     } gap;
 
+    // NOLINTNEXTLINE(readability-identifier-naming)
     struct gattc_event {
       esp_gattc_cb_event_t gattc_event;
       esp_gatt_if_t gattc_if;
@@ -113,6 +119,7 @@ class BLEEvent {
       uint8_t data[64];
     } gattc;
 
+    // NOLINTNEXTLINE(readability-identifier-naming)
     struct gatts_event {
       esp_gatts_cb_event_t gatts_event;
       esp_gatt_if_t gatts_if;
@@ -120,6 +127,7 @@ class BLEEvent {
       uint8_t data[64];
     } gatts;
   } event_;
+  // NOLINTNEXTLINE(readability-identifier-naming)
   enum ble_event_t : uint8_t {
     GAP,
     GATTC,
