@@ -5,22 +5,22 @@ from esphome.components import mqtt, power_supply
 from esphome.const import (
     CONF_COLOR_CORRECT,
     CONF_DEFAULT_TRANSITION_LENGTH,
-    CONF_DISABLED_BY_DEFAULT,
     CONF_EFFECTS,
+    CONF_FLASH_TRANSITION_LENGTH,
     CONF_GAMMA_CORRECT,
     CONF_ID,
-    CONF_INTERNAL,
-    CONF_NAME,
     CONF_MQTT_ID,
     CONF_POWER_SUPPLY,
     CONF_RESTORE_MODE,
     CONF_ON_TURN_OFF,
     CONF_ON_TURN_ON,
+    CONF_ON_STATE,
     CONF_TRIGGER_ID,
     CONF_COLD_WHITE_COLOR_TEMPERATURE,
     CONF_WARM_WHITE_COLOR_TEMPERATURE,
 )
 from esphome.core import coroutine_with_priority
+from esphome.cpp_helpers import setup_entity
 from .automation import light_control_to_code  # noqa
 from .effects import (
     validate_effects,
@@ -38,6 +38,7 @@ from .types import (  # noqa
     AddressableLight,
     LightTurnOnTrigger,
     LightTurnOffTrigger,
+    LightStateTrigger,
 )
 
 CODEOWNERS = ["@esphome/core"]
@@ -53,7 +54,7 @@ RESTORE_MODES = {
     "RESTORE_INVERTED_DEFAULT_ON": LightRestoreMode.LIGHT_RESTORE_INVERTED_DEFAULT_ON,
 }
 
-LIGHT_SCHEMA = cv.NAMEABLE_SCHEMA.extend(cv.MQTT_COMMAND_COMPONENT_SCHEMA).extend(
+LIGHT_SCHEMA = cv.ENTITY_BASE_SCHEMA.extend(cv.MQTT_COMMAND_COMPONENT_SCHEMA).extend(
     {
         cv.GenerateID(): cv.declare_id(LightState),
         cv.OnlyWith(CONF_MQTT_ID, "mqtt"): cv.declare_id(mqtt.MQTTJSONLightComponent),
@@ -70,6 +71,11 @@ LIGHT_SCHEMA = cv.NAMEABLE_SCHEMA.extend(cv.MQTT_COMMAND_COMPONENT_SCHEMA).exten
                 cv.GenerateID(CONF_TRIGGER_ID): cv.declare_id(LightTurnOffTrigger),
             }
         ),
+        cv.Optional(CONF_ON_STATE): auto.validate_automation(
+            {
+                cv.GenerateID(CONF_TRIGGER_ID): cv.declare_id(LightStateTrigger),
+            }
+        ),
     }
 )
 
@@ -85,6 +91,9 @@ BRIGHTNESS_ONLY_LIGHT_SCHEMA = LIGHT_SCHEMA.extend(
         cv.Optional(
             CONF_DEFAULT_TRANSITION_LENGTH, default="1s"
         ): cv.positive_time_period_milliseconds,
+        cv.Optional(
+            CONF_FLASH_TRANSITION_LENGTH, default="0s"
+        ): cv.positive_time_period_milliseconds,
         cv.Optional(CONF_EFFECTS): validate_effects(MONOCHROMATIC_EFFECTS),
     }
 )
@@ -122,16 +131,20 @@ def validate_color_temperature_channels(value):
 
 
 async def setup_light_core_(light_var, output_var, config):
-    cg.add(light_var.set_disabled_by_default(config[CONF_DISABLED_BY_DEFAULT]))
+    await setup_entity(light_var, config)
+
     cg.add(light_var.set_restore_mode(config[CONF_RESTORE_MODE]))
-    if CONF_INTERNAL in config:
-        cg.add(light_var.set_internal(config[CONF_INTERNAL]))
+
     if CONF_DEFAULT_TRANSITION_LENGTH in config:
         cg.add(
             light_var.set_default_transition_length(
                 config[CONF_DEFAULT_TRANSITION_LENGTH]
             )
         )
+    if CONF_FLASH_TRANSITION_LENGTH in config:
+        cg.add(
+            light_var.set_flash_transition_length(config[CONF_FLASH_TRANSITION_LENGTH])
+        )
     if CONF_GAMMA_CORRECT in config:
         cg.add(light_var.set_gamma_correct(config[CONF_GAMMA_CORRECT]))
     effects = await cg.build_registry_list(
@@ -145,6 +158,9 @@ async def setup_light_core_(light_var, output_var, config):
     for conf in config.get(CONF_ON_TURN_OFF, []):
         trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], light_var)
         await auto.build_automation(trigger, [], conf)
+    for conf in config.get(CONF_ON_STATE, []):
+        trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], light_var)
+        await auto.build_automation(trigger, [], conf)
 
     if CONF_COLOR_CORRECT in config:
         cg.add(output_var.set_correction(*config[CONF_COLOR_CORRECT]))
@@ -159,7 +175,7 @@ async def setup_light_core_(light_var, output_var, config):
 
 
 async def register_light(output_var, config):
-    light_var = cg.new_Pvariable(config[CONF_ID], config[CONF_NAME], output_var)
+    light_var = cg.new_Pvariable(config[CONF_ID], output_var)
     cg.add(cg.App.register_light(light_var))
     await cg.register_component(light_var, config)
     await setup_light_core_(light_var, output_var, config)
