@@ -1,6 +1,8 @@
 #include "mqtt_light.h"
 #include "esphome/core/log.h"
 
+#include "mqtt_const.h"
+
 #ifdef USE_MQTT
 #ifdef USE_LIGHT
 
@@ -16,7 +18,7 @@ std::string MQTTJSONLightComponent::component_type() const { return "light"; }
 const EntityBase *MQTTJSONLightComponent::get_entity() const { return this->state_; }
 
 void MQTTJSONLightComponent::setup() {
-  this->subscribe_json(this->get_command_topic_(), [this](const std::string &topic, JsonObject &root) {
+  this->subscribe_json(this->get_command_topic_(), [this](const std::string &topic, JsonObject root) {
     LightCall call = this->state_->make_call();
     LightJSONSchema::parse_json(*this->state_, call, root);
     call.perform();
@@ -30,16 +32,16 @@ MQTTJSONLightComponent::MQTTJSONLightComponent(LightState *state) : MQTTComponen
 
 bool MQTTJSONLightComponent::publish_state_() {
   return this->publish_json(this->get_state_topic_(),
-                            [this](JsonObject &root) { LightJSONSchema::dump_json(*this->state_, root); });
+                            [this](JsonObject root) { LightJSONSchema::dump_json(*this->state_, root); });
 }
 LightState *MQTTJSONLightComponent::get_state() const { return this->state_; }
 
-void MQTTJSONLightComponent::send_discovery(JsonObject &root, mqtt::SendDiscoveryConfig &config) {
+void MQTTJSONLightComponent::send_discovery(JsonObject root, mqtt::SendDiscoveryConfig &config) {
   root["schema"] = "json";
   auto traits = this->state_->get_traits();
 
-  root["color_mode"] = true;
-  JsonArray &color_modes = root.createNestedArray("supported_color_modes");
+  root[MQTT_COLOR_MODE] = true;
+  JsonArray color_modes = root.createNestedArray("supported_color_modes");
   if (traits.supports_color_mode(ColorMode::ON_OFF))
     color_modes.add("onoff");
   if (traits.supports_color_mode(ColorMode::BRIGHTNESS))
@@ -64,7 +66,7 @@ void MQTTJSONLightComponent::send_discovery(JsonObject &root, mqtt::SendDiscover
 
   if (this->state_->supports_effects()) {
     root["effect"] = true;
-    JsonArray &effect_list = root.createNestedArray("effect_list");
+    JsonArray effect_list = root.createNestedArray(MQTT_EFFECT_LIST);
     for (auto *effect : this->state_->get_effects())
       effect_list.add(effect->get_name());
     effect_list.add("None");
