@@ -1,20 +1,18 @@
 #include "mqtt_text_sensor.h"
 #include "esphome/core/log.h"
 
+#ifdef USE_MQTT
 #ifdef USE_TEXT_SENSOR
 
 namespace esphome {
 namespace mqtt {
 
-static const char *TAG = "mqtt.text_sensor";
+static const char *const TAG = "mqtt.text_sensor";
 
 using namespace esphome::text_sensor;
 
 MQTTTextSensor::MQTTTextSensor(TextSensor *sensor) : MQTTComponent(), sensor_(sensor) {}
-void MQTTTextSensor::send_discovery(JsonObject &root, mqtt::SendDiscoveryConfig &config) {
-  if (!this->sensor_->get_icon().empty())
-    root["icon"] = this->sensor_->get_icon();
-
+void MQTTTextSensor::send_discovery(JsonObject root, mqtt::SendDiscoveryConfig &config) {
   config.command_topic = false;
 }
 void MQTTTextSensor::setup() {
@@ -34,12 +32,12 @@ bool MQTTTextSensor::send_initial_state() {
     return true;
   }
 }
-bool MQTTTextSensor::is_internal() { return this->sensor_->is_internal(); }
 std::string MQTTTextSensor::component_type() const { return "sensor"; }
-std::string MQTTTextSensor::friendly_name() const { return this->sensor_->get_name(); }
+const EntityBase *MQTTTextSensor::get_entity() const { return this->sensor_; }
 std::string MQTTTextSensor::unique_id() { return this->sensor_->unique_id(); }
 
 }  // namespace mqtt
 }  // namespace esphome
 
 #endif
+#endif  // USE_MQTT
