@@ -1,8 +1,9 @@
 #pragma once
 
-#include <queue>
 #include "esphome/core/component.h"
 #include "esphome/core/helpers.h"
+#include <queue>
+#include <utility>
 
 namespace esphome {
 namespace sensor {
@@ -32,11 +33,6 @@ class Filter {
 
   void input(float value);
 
-  /// Return the amount of time that this filter is expected to take based on the input time interval.
-  virtual uint32_t expected_interval(uint32_t input);
-
-  uint32_t calculate_remaining_interval(uint32_t input);
-
   void output(float value);
 
  protected:
@@ -46,6 +42,37 @@ class Filter {
   Sensor *parent_{nullptr};
 };
 
+/** Simple quantile filter.
+ *
+ * Takes the quantile of the last <send_every> values and pushes it out every <send_every>.
+ */
+class QuantileFilter : public Filter {
+ public:
+  /** Construct a QuantileFilter.
+   *
+   * @param window_size The number of values that should be used in quantile calculation.
+   * @param send_every After how many sensor values should a new one be pushed out.
+   * @param send_first_at After how many values to forward the very first value. Defaults to the first value
+   *   on startup being published on the first *raw* value, so with no filter applied. Must be less than or equal to
+   *   send_every.
+   * @param quantile float 0..1 to pick the requested quantile. Defaults to 0.9.
+   */
+  explicit QuantileFilter(size_t window_size, size_t send_every, size_t send_first_at, float quantile);
+
+  optional<float> new_value(float value) override;
+
+  void set_send_every(size_t send_every);
+  void set_window_size(size_t window_size);
+  void set_quantile(float quantile);
+
+ protected:
+  std::deque<float> queue_;
+  size_t send_every_;
+  size_t send_at_;
+  size_t window_size_;
+  float quantile_;
+};
+
 /** Simple median filter.
  *
  * Takes the median of the last <send_every> values and pushes it out every <send_every>.
@@ -67,8 +94,6 @@ class MedianFilter : public Filter {
   void set_send_every(size_t send_every);
   void set_window_size(size_t window_size);
 
-  uint32_t expected_interval(uint32_t input) override;
-
  protected:
   std::deque<float> queue_;
   size_t send_every_;
@@ -97,8 +122,6 @@ class MinFilter : public Filter {
   void set_send_every(size_t send_every);
   void set_window_size(size_t window_size);
 
-  uint32_t expected_interval(uint32_t input) override;
-
  protected:
   std::deque<float> queue_;
   size_t send_every_;
@@ -127,8 +150,6 @@ class MaxFilter : public Filter {
   void set_send_every(size_t send_every);
   void set_window_size(size_t window_size);
 
-  uint32_t expected_interval(uint32_t input) override;
-
  protected:
   std::deque<float> queue_;
   size_t send_every_;
@@ -158,8 +179,6 @@ class SlidingWindowMovingAverageFilter : public Filter {
   void set_send_every(size_t send_every);
   void set_window_size(size_t window_size);
 
-  uint32_t expected_interval(uint32_t input) override;
-
  protected:
   float sum_{0.0};
   std::deque<float> queue_;
@@ -182,8 +201,6 @@ class ExponentialMovingAverageFilter : public Filter {
   void set_send_every(size_t send_every);
   void set_alpha(float alpha);
 
-  uint32_t expected_interval(uint32_t input) override;
-
  protected:
   bool first_value_{true};
   float accumulator_{0.0f};
@@ -192,6 +209,26 @@ class ExponentialMovingAverageFilter : public Filter {
   float alpha_;
 };
 
+/** Simple throttle average filter.
+ *
+ * It takes the average of all the values received in a period of time.
+ */
+class ThrottleAverageFilter : public Filter, public Component {
+ public:
+  explicit ThrottleAverageFilter(uint32_t time_period);
+
+  void setup() override;
+
+  optional<float> new_value(float value) override;
+
+  float get_setup_priority() const override;
+
+ protected:
+  uint32_t time_period_;
+  float sum_{0.0f};
+  unsigned int n_{0};
+};
+
 using lambda_filter_t = std::function<optional<float>(float)>;
 
 /** This class allows for creation of simple template filters.
@@ -278,8 +315,6 @@ class HeartbeatFilter : public Filter, public Component {
 
   optional<float> new_value(float value) override;
 
-  uint32_t expected_interval(uint32_t input) override;
-
   float get_setup_priority() const override;
 
  protected:
@@ -305,8 +340,6 @@ class OrFilter : public Filter {
 
   void initialize(Sensor *parent, Filter *next) override;
 
-  uint32_t expected_interval(uint32_t input) override;
-
   optional<float> new_value(float value) override;
 
  protected:
@@ -335,7 +368,7 @@ class CalibrateLinearFilter : public Filter {
 
 class CalibratePolynomialFilter : public Filter {
  public:
-  CalibratePolynomialFilter(const std::vector<float> &coefficients) : coefficients_(coefficients) {}
+  CalibratePolynomialFilter(std::vector<float> coefficients) : coefficients_(std::move(coefficients)) {}
   optional<float> new_value(float value) override;
 
  protected:
