@@ -1,6 +1,6 @@
 """Constants used by esphome."""
 
-__version__ = "2021.12.0"
+__version__ = "2022.1.1"
 
 ALLOWED_NAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789-_"
 
@@ -402,6 +402,7 @@ CONF_NUM_CHIPS = "num_chips"
 CONF_NUM_LEDS = "num_leds"
 CONF_NUM_SCANS = "num_scans"
 CONF_NUMBER = "number"
+CONF_NUMBER_DATAPOINT = "number_datapoint"
 CONF_OFF_MODE = "off_mode"
 CONF_OFFSET = "offset"
 CONF_ON = "on"
@@ -518,6 +519,7 @@ CONF_PULLDOWN = "pulldown"
 CONF_PULLUP = "pullup"
 CONF_PULSE_LENGTH = "pulse_length"
 CONF_QOS = "qos"
+CONF_QUANTILE = "quantile"
 CONF_RADON = "radon"
 CONF_RADON_LONG_TERM = "radon_long_term"
 CONF_RANDOM = "random"
@@ -539,6 +541,7 @@ CONF_REFERENCE_TEMPERATURE = "reference_temperature"
 CONF_REFRESH = "refresh"
 CONF_REPEAT = "repeat"
 CONF_REPOSITORY = "repository"
+CONF_RESET_DURATION = "reset_duration"
 CONF_RESET_PIN = "reset_pin"
 CONF_RESIZE = "resize"
 CONF_RESOLUTION = "resolution"
@@ -596,6 +599,7 @@ CONF_SHOW_VALUES = "show_values"
 CONF_SHUNT_RESISTANCE = "shunt_resistance"
 CONF_SHUNT_VOLTAGE = "shunt_voltage"
 CONF_SHUTDOWN_MESSAGE = "shutdown_message"
+CONF_SIGNAL_STRENGTH = "signal_strength"
 CONF_SINGLE_LIGHT_ID = "single_light_id"
 CONF_SIZE = "size"
 CONF_SLEEP_DURATION = "sleep_duration"
@@ -681,6 +685,7 @@ CONF_TOLERANCE = "tolerance"
 CONF_TOPIC = "topic"
 CONF_TOPIC_PREFIX = "topic_prefix"
 CONF_TOTAL = "total"
+CONF_TOTAL_POWER = "total_power"
 CONF_TRACES = "traces"
 CONF_TRANSITION_LENGTH = "transition_length"
 CONF_TRIGGER_ID = "trigger_id"
@@ -831,6 +836,7 @@ UNIT_MINUTE = "min"
 UNIT_OHM = "Ω"
 UNIT_PARTS_PER_BILLION = "ppb"
 UNIT_PARTS_PER_MILLION = "ppm"
+UNIT_PASCAL = "Pa"
 UNIT_PERCENT = "%"
 UNIT_PULSES = "pulses"
 UNIT_PULSES_PER_MINUTE = "pulses/min"
